#include "bitboard.hh"

void BitBoard::init() {
	for(int i = 0; i < 28; i++) {
		TempBitBoard t;
		for(int j = 0; j < 5; j++) {
			t.setBits(_fin[i][j]);
		}
		fin[i] = t.getBits();
	}

	for(int i=0; i < 3; i++) {
		for(int j=0; j < 4; j++) {
			TempBitBoard t;
			for(int k = 0; k < 9; k++) {
				if(_bit_add_liste[i][j][k] != -1) t.setBits(_bit_add_liste[i][j][k]);
			}
			bit_add_liste[i][j] = t.getBits();
		}
	}

	for(int i = 0; i < 4; i++) {
		TempBitBoard t;
		for(int j=0; j < 9; j++) {
			if(j!=4) t.setBits(bit_ligne[i][j]);		
		}
		mask[i] =~ t.getBits();
	}
	
}

BitBoard::BitBoard() {
	_bits[0] = _bits[1] = 0;
	_player[0] = _player[1] = 0;
	_display[0] = _display[1] = _display[2] = _display[3] = _display[4] = 0;
	_key = 0;
	init();
}


uint64_t BitBoard::getKey() {
	return _key;
	//return (_bits[0]|_bits[1]);
}


bool BitBoard::gameover(const bool& __couleur) {
	if(getPlayer(__couleur)<5) return false;

	uint64_t temp;

	if(__couleur) temp = _bits[0];
	else temp = _bits[1];

	for(int i = 0; i < 28; i++) {
		if( (temp&fin[i]) == fin[i] ) return true;
	}
	return false;
}

void BitBoard::turn(const int& __tab, const bool& __t) {
	if(__t) turnRight(__tab);
	else turnLeft(__tab);
}

inline void BitBoard::turnRight(const int& __i) {

	uint64_t temp[2];
	temp[0] = _bits[0];
	temp[1] = _bits[1];

	_bits[0] &= mask[__i];
	_bits[1] &= mask[__i];

	for(int i = 0; i < 9; i++) {;
		if(i!=4) {
			if(temp[0] & (((uint64_t)1) << bit_ligne[__i][i])) {
				_key ^= _zobrist.getKey(bit_ligne[__i][i], bit_tab_display[bit_ligne[__i][i]], 0);
				_key ^= _zobrist.getKey(bit_turnRight[__i][i], bit_tab_display[bit_turnRight[__i][i]], 0);
				_bits[0] |=  (((uint64_t) 1) << bit_turnRight[__i][i]);
			}
			else {
				if(temp[1] & (((uint64_t)1) << bit_ligne[__i][i])) {
					_key ^= _zobrist.getKey(bit_ligne[__i][i], bit_tab_display[bit_ligne[__i][i]], 1);
					_key ^= _zobrist.getKey(bit_turnRight[__i][i], bit_tab_display[bit_turnRight[__i][i]], 1);
					_bits[1] |=  (((uint64_t) 1) << bit_turnRight[__i][i]);
				}
			}
		}
	}

}

inline void BitBoard::turnLeft(const int& __i) {
	uint64_t temp[2];
	temp[0] = _bits[0];
	temp[1] = _bits[1];

	_bits[0] &= mask[__i];
	_bits[1] &= mask[__i];

	for(int i = 0; i < 9; i++) {
		if(i!=4) {
			if(temp[0] & (((uint64_t)1) << bit_ligne[__i][i])) {
				_key ^= _zobrist.getKey(bit_ligne[__i][i], bit_tab_display[bit_ligne[__i][i]], 0);
				_key ^= _zobrist.getKey(bit_turnLeft[__i][i], bit_tab_display[bit_turnLeft[__i][i]], 0);
				_bits[0] |=  (((uint64_t) 1) << bit_turnLeft[__i][i]);
			}
			else {
				if(temp[1] & (((uint64_t)1) << bit_ligne[__i][i])) {
					_key ^= _zobrist.getKey(bit_ligne[__i][i], bit_tab_display[bit_ligne[__i][i]], 1);
					_key ^= _zobrist.getKey(bit_turnLeft[__i][i], bit_tab_display[bit_turnLeft[__i][i]], 1);
					_bits[1] |=  (((uint64_t) 1) << bit_turnLeft[__i][i]);
				}
			}
		}
	}
}


void BitBoard::display() {
	for(int i = 36; i != 0; i--) {
		if(!(i%6)) cout<<endl;
		if (_bits[0] & (((uint64_t)1) << (i-1))) cout<<"1 ";
		else if (_bits[1] & (((uint64_t)1) << (i-1))) cout<<"2 ";
		else cout<<"0 ";
	}
	cout<<endl<<"Key : "<<_key<<endl;
	cout<<endl<<endl;
}

void BitBoard::setPlayer(const int& __i, const bool& __couleur) {

	int temp = bit_tab_display[__i];

	_display[temp]++;

	if(__couleur) {
		_bits[0] |=  (((uint64_t) 1) << __i);
		_player[0]++;
		_key ^= _zobrist.getKey(__i, temp, 0);
	}
	else {
		_bits[1] |=  (((uint64_t) 1) << __i);
		_player[1]++;
		_key ^= _zobrist.getKey(__i, temp, 1);
	}

}

void BitBoard::erasePlayer(const int& __i) {

	int temp = bit_tab_display[__i];

	if(_bits[0] & (((uint64_t)1) << __i)) {
		_bits[0] &= ~(((uint64_t)1) << __i);
		_key ^= _zobrist.getKey(__i, temp, 0);
		_player[0]--;
		_display[temp]--;
	}
	else {
		if(_bits[1] & (((uint64_t)1) << __i)) {
			_bits[1] &= ~(((uint64_t)1) << __i);
			_key ^= _zobrist.getKey(__i, temp, 1);
			_player[1]--;
			_display[temp]--;
		}
	}
}


bool BitBoard::getPlayer(const int& __i, const bool& __couleur) {
	if(__couleur) return (_bits[0] & (((uint64_t)1) << __i));
	else return (_bits[1] & (((uint64_t)1) << __i));
}

Liste* BitBoard::getListe() {
	uint64_t temp = ~(_bits[0]|_bits[1]);
	Liste* t = new Liste();
	for(int i = 35; i != -1; i--) {
		if(temp & (((uint64_t)1) << i)) {
			for(int e = 0; e < 4; e++) {
				t->add(i, e, true);
				if(addListe(e,i)==true) t->add(i, e, false);
			}
		}
	}
	return t;
}

int BitBoard::getPlayer(const bool& __couleur) {
	if(__couleur) return _player[0];
	else return _player[1];
}


bool BitBoard::addListe(const int& __e, const int& __i) {
/*
	if(bit_tab_display[__i] != __e) {
		if(_display[__e] == 0) return false;
		else if(_display[__e] == 1 && diago(__e)) return false;
		else if(_display[__e] == 4 && carre(__e)) return false;
		else if(_display[__e] == 5 && croix(__e)) return false;
		else if(_display[__e] == 9 && rempli(__e)) return false;
		else return true;
	}
*/
	return true;
}

/*0 0 0
  0 X 0
  0 0 0*/
inline bool BitBoard::diago(const int& __e) {
	if( (_bits[0]&bit_add_liste[0][__e]) == bit_add_liste[0][__e] ) return true;
	if( (_bits[1]&bit_add_liste[0][__e]) == bit_add_liste[0][__e] ) return true;
	return false;
}

/*X 0 X
  0 0 0
  X 0 X*/
inline bool BitBoard::carre(const int& __e) {
	if( (_bits[0]&bit_add_liste[1][__e]) == bit_add_liste[1][__e] ) return true;
	if( (_bits[1]&bit_add_liste[1][__e]) == bit_add_liste[1][__e] ) return true;
	return false;
}

/*X X X
  X X X
  X X X*/
inline bool BitBoard::rempli(const int& __e) {
	return false;
}


/*0 X 0
  X X X
  0 X 0*/
inline bool BitBoard::croix(const int& __e) {
	if( (_bits[0]&bit_add_liste[2][__e]) == bit_add_liste[2][__e] ) return true;
	if( (_bits[1]&bit_add_liste[2][__e]) == bit_add_liste[2][__e] ) return true;
	return false;
}
