#ifndef BITBOARD
#define BITBOARD

#include <iostream>
#include "liste.hh"
#include "board_arbitrary.hh"
#include "zobrist.hh"

using namespace std;

static const int _bit_add_liste[3][4][9] = {
	{
		{28, -1, -1, -1, -1, -1, -1, -1, -1},
		{25, -1, -1, -1, -1, -1, -1, -1, -1},
		{10, -1, -1, -1, -1, -1, -1, -1, -1},
		{7, -1, -1, -1, -1, -1, -1, -1, -1}
	},
	{
		{35, 33, 23, 21, -1, -1, -1, -1, -1},
		{32, 30, 20, 18, -1, -1, -1, -1, -1},
		{17, 15, 5, 3, -1, -1, -1, -1, -1},
		{14, 12, 2, 0, -1, -1, -1, -1, -1}
	},
	{
		{34, 29, 28, 27, 22, -1, -1, -1, -1},
		{31, 26, 25, 24, 19, -1, -1, -1, -1},
		{16, 11, 10, 9, 4, -1, -1, -1, -1},
		{13, 8, 7, 6, 1, -1, -1, -1, -1}
	}
};

static const int _fin[28][5] = {
	{ 35, 34, 33, 32, 31 },
	{ 34, 33, 32, 31, 30 },
	{ 29, 28, 27, 26, 25 },
	{ 28, 27, 26, 25, 24 },
	{ 23, 22, 21, 20, 19 },
	{ 22, 21, 20, 19, 18 },
	{ 17, 16, 15, 14, 13 },
	{ 16, 15, 14, 13, 12 },
	{ 11, 10, 9, 8, 7 },
	{ 10, 9, 8, 7, 6 },
	{ 5, 4, 3, 2, 1 },
	{ 4, 3, 2, 1, 0 },
	{ 35, 29, 23, 17, 11 },
	{ 34, 28, 22, 16, 10 },
	{ 33, 27, 21, 15, 9 },
	{ 32, 26, 20, 14, 8 },
	{ 31, 25, 19, 13, 7 },
	{ 30, 24, 18, 12, 6 },
	{ 29, 23, 17, 11, 5 },
	{ 28, 22, 16, 10, 4 },
	{ 27, 21, 15, 9, 3 },
	{ 26, 20, 14, 8, 2 },
	{ 25, 19, 13, 7, 1 },
	{ 24, 18, 12, 6, 0 },
	{ 35, 28, 21, 14, 7 },
	{ 28, 21, 14, 7, 0 },
	{ 34, 27, 20, 13, 6 },
	{ 29, 22, 15, 8, 1 }
};

static const int bit_tab_display[] =  { 3, 3, 3, 2, 2, 2, 3, 3, 3, 2, 2, 2, 3, 3, 3, 2, 2, 2, 1, 1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0 };

static const int bit_ligne[4][9] = {{ 35, 34, 33, 29, 28, 27, 23, 22, 21 }, { 32, 31, 30, 26, 25, 24, 20, 19, 18 }, { 17, 16, 15, 11, 10, 9, 5, 4, 3 }, { 14, 13, 12, 8, 7, 6, 2, 1, 0 } };

static const int bit_turnRight[4][9] = {{33, 27, 21, 34, 28, 22, 35, 29, 23 }, { 30, 24, 18, 31, 25, 19, 32, 26, 20}, { 15, 9, 3, 16, 10, 4, 17, 11, 5 }, { 12, 6, 0, 13, 7, 1, 14, 8, 2} };

static const int bit_turnLeft[4][9] = {{ 23, 29, 35, 22, 28, 34, 21, 27, 33}, { 20, 26, 32, 19, 25, 31, 18, 24, 30 }, { 5, 11, 17, 4, 10, 16, 3, 9, 15}, {2, 8, 14, 1, 7, 13, 0, 6, 12 } };

class TempBitBoard {
	private:
		uint64_t _bits;

	public:
		TempBitBoard() : _bits(0) { }
		inline void setBits(unsigned int __bits) {
			_bits |= (((uint64_t) 1) << __bits);
		}
		inline void display() {
			cout<<_bits<<endl;
		}
		inline uint64_t getBits() { return _bits; }
};

class BitBoard : public BoardArbitrary {

	private:
		Zobrist _zobrist;
		uint64_t _key;

	private:
		uint64_t _bits[2];
		int _player[2];
		int _display[4];

	private:
		uint64_t bit_add_liste[3][4];
		uint64_t fin[28];
		uint64_t mask[4];

	public:
		BitBoard();
		
		uint64_t getKey();
		bool gameover(const bool&);
		void turn(const int&, const bool&);
		void display();
		void setPlayer(const int&, const bool&);
		void erasePlayer(const int&);
		bool getPlayer(const int&, const bool&);
		Liste* getListe();
		int getPlayer(const bool&);

		inline int getPlayer(const int&);

	private:
		void init();

		inline void turnRight(const int&);
		inline void turnLeft(const int&);

		bool addListe(const int&, const int&);
		inline bool diago(const int&);
		inline bool carre(const int&);
		inline bool rempli(const int&);
		inline bool croix(const int&);
};


#endif
