#include "board.hh"


int Board::getPlayer(const bool& __couleur) {
	if(__couleur) return _player[0];
	else return _player[1];
}

void Board::setPlayer(const int& __i, const bool& __couleur) {
	if(__couleur) {
		_matrice[__i] = 1;
		_player[0] = _player[0]+1;
		_key ^= _zobrist.getKey(__i, tab_display[__i], 0);
		_display[tab_display[__i]] = _display[tab_display[__i]] + 1;
	}
	else {
		_matrice[__i] = 2;
		_player[1] = _player[1]+1;
		_key ^= _zobrist.getKey(__i, tab_display[__i], 1);
		_display[tab_display[__i]] = _display[tab_display[__i]] + 1;
	}
}

void Board::erasePlayer(const int& __i) {
	if(_matrice[__i]==1) {
		_key ^=_zobrist.getKey(__i, tab_display[__i], 0);
		_player[0] = _player[0] -1;
		_display[tab_display[__i]] = _display[tab_display[__i]] - 1;
	}
	else if(_matrice[__i]==2) {
		_key ^=_zobrist.getKey(__i, tab_display[__i], 1);
		_player[1] = _player[1] - 1;
		_display[tab_display[__i]] = _display[tab_display[__i]] - 1;
	}
	else { }
	_matrice[__i] = 0;
}

int Board::getPlayer(const int& __i) { return _matrice[__i]; }

bool Board::getPlayer(const int& __i, const bool& __couleur) {
	int couleur;
	if(__couleur) couleur = 1;
	else couleur = 2;

	return (couleur == getPlayer(__i));
}

void Board::turn(const int& __tab, const bool& __t) {
	if(__t) turnRight(__tab);
	else turnLeft(__tab);
}

bool Board::gameover(const bool& __couleur) {

	if(getPlayer(__couleur)<5) return false;
	
	for(int i = 0; i < 12; i++) {
		if(gameoverColonne(i, __couleur) == true) return true;
	}

	for(int i = 0; i < 32; i=i+6) {
		for(int j = i; j < i+2; j++) {
			if(gameoverLigne(j, __couleur) == true) return true;
		}
	}

	for(int i = 0; i < 8; i=i+6) {
		for(int j = i; j < i+2; j++) {
			if(gameoverDiagonale(j, __couleur) == true) return true;
		}
	}

	return false;
}



bool Board::gameoverLigne(const int& __i, const bool& __couleur) {
	int couleur;
	if(__couleur) couleur = 1;
	else couleur = 2;

	for(int i = __i; i < __i+5; i++) {
		if(_matrice[i] != couleur) return false;
	}
	return true;
}

bool Board::gameoverColonne(const int& __i, const bool& __couleur) {
	int couleur;
	if(__couleur) couleur = 1;
	else couleur = 2;

	for(int i = __i; i < (6*5)+__i; i = i +6) {
		if(_matrice[i] != couleur) return false;
	}
	return true;
}


bool Board::gameoverDiagonale(const int& __i, const bool& __couleur) {
	int couleur;
	if(__couleur) couleur = 1;
	else couleur = 2;

	for(int i = __i; i < __i+(5*7); i = i+7) {
		if(_matrice[i] != couleur) return false;
	}
	return true;
}

Board::Board() {
	for(int i = 0; i < 36; i++) {
		_matrice[i] = 0;
	}

	for(int i = 0; i < 2; i++) {
		_player[i] = 0;
	}

	for(int i=0; i < 4; i++) {
		_display[i] = 0;
	}

	_key = 0;
}

void Board::turnLeft(const int& __i) {
	//On copie les resultats
	int temp[9];
	for(int i = 0; i < 9; i++) {
		temp[i] = _matrice[ligne[__i][i]];

		if(temp[i] == 1) _key ^= _zobrist.getKey(ligne[__i][i], tab_display[ligne[__i][i]], 0);
		else if(temp[i] == 2) _key ^= _zobrist.getKey(ligne[__i][i], tab_display[ligne[__i][i]], 1);
		else { }
	}

	//On inverse les matrices
	int t = 0;

	for(int i = 2; i > -1; i--) {
		for(int j=i; j < 9; j = j+3) {
			_matrice[ligne[__i][t]] = temp[j];

			if(_matrice[ligne[__i][t]] == 1) _key ^= _zobrist.getKey(ligne[__i][t], tab_display[ligne[__i][t]], 0);
			else if(_matrice[ligne[__i][t]] == 2) _key ^= _zobrist.getKey(ligne[__i][t], tab_display[ligne[__i][t]], 1);
			else { }

			t++;
		}
	}
}

void Board::turnRight(const int& __i) {
	//On copie les resultats
	int temp[9];
	for(int i = 0; i < 9; i++) {
		temp[i] = _matrice[ligne[__i][i]];

		if(temp[i] == 1) _key ^= _zobrist.getKey(ligne[__i][i], tab_display[ligne[__i][i]], 0);
		else if(temp[i] == 2) _key ^= _zobrist.getKey(ligne[__i][i], tab_display[ligne[__i][i]], 1);
		else { }
	}

	//On inverse les matrices
	int t = 0;

	for(int i = 6; i < 9; i++) {
		for(int j=i; j > -1; j = j-3) {
			_matrice[ligne[__i][t]] = temp[j];

			if(_matrice[ligne[__i][t]] == 1) _key ^= _zobrist.getKey(ligne[__i][t], tab_display[ligne[__i][t]], 0);
			else if(_matrice[ligne[__i][t]] == 2) _key ^= _zobrist.getKey(ligne[__i][t], tab_display[ligne[__i][t]], 1);
			else { }

			t++;
		}
	}

}

/*0 0 0
  0 X 0
  0 0 0*/
bool Board::diago(const int& __e) {
	if(_matrice[ligne[__e][4]] != 0) return true;
	return false;
}

/*X 0 X
  0 0 0
  X 0 X*/
bool Board::carre(const int& __e) {
	if((_matrice[ligne[__e][0]] != 0) && (_matrice[ligne[__e][0]] == _matrice[ligne[__e][2]] && _matrice[ligne[__e][2]] == _matrice[ligne[__e][6]] && _matrice[ligne[__e][6]] == _matrice[ligne[__e][8]])) return true;
	return false;
}

/*0 X 0
  X X X
  0 X 0*/
bool Board::croix(const int& __e) {
	if((_matrice[ligne[__e][1]] != 0) && (_matrice[ligne[__e][1]] == _matrice[ligne[__e][3]] && _matrice[ligne[__e][3]] == _matrice[ligne[__e][4]] && _matrice[ligne[__e][4]] == _matrice[ligne[__e][5]] && _matrice[ligne[__e][5]] == _matrice[ligne[__e][7]])) return true;
	return false;
}

/*X X X
  X X X
  X X X*/
bool Board::rempli(const int& __e) {
	int old = _matrice[ligne[__e][0]];
	for(int i = 1; i < 9; i++) {
		if(old != _matrice[ligne[__e][i]]) return false;
	}
	return true;
}

bool Board::addListe(const int& __e, const int& __i) {
/*
	if(tab_display[__i] != __e) {
		if(_display[__e] == 0) return false;
		else if(_display[__e] == 1 && diago(__e)) return false;
		else if(_display[__e] == 4 && carre(__e)) return false;
		 else if(_display[__e] == 5 && croix(__e)) return false;
		else if(_display[__e] == 9 && rempli(__e)) return false;
		else return true;
	}
*/
	return true;
}

Liste* Board::getListe() {
	Liste* t = new Liste();
	for(int i = 0; i < 36; i++) {
		if(_matrice[i] == 0) {
			for(int e = 0; e < 4; e++) {
				t->add(i, e, true);
				if(addListe(e,i)==true) t->add(i, e, false);
			}
		}
	}
	return t;
}

void Board::display() {
	for(int i = 0; i < 36; i++) {
		if(!(i%6)) cout<<endl;
		cout<<_matrice[i]<<" ";
	}
	cout<<endl<<"Key : "<<_key<<endl;
	cout<<endl<<endl;
}
