#include "engine.hh"

Engine::Engine() {
	_game = new BitBoard2();
}

void Engine::set_title(const string& __title) {
	SDL_WM_SetCaption( __title.c_str(), NULL );
}

void Engine::init() {
	SDL_Init(SDL_INIT_VIDEO);
	_screen = SDL_SetVideoMode(280, 280, 32, SDL_HWSURFACE | SDL_DOUBLEBUF);
	init_game();
}

void Engine::init_game() {
	_image[0] = SDL_LoadBMP("./pics/q1ccw.bmp");
	_image[1] = SDL_LoadBMP("./pics/q1cw.bmp");
	_image[2] = SDL_LoadBMP("./pics/q2ccw.bmp");
	_image[3] = SDL_LoadBMP("./pics/q2cw.bmp");
	_image[4] = SDL_LoadBMP("./pics/q4ccw.bmp");
	_image[5] = SDL_LoadBMP("./pics/q4cw.bmp");
	_image[6] = SDL_LoadBMP("./pics/q3ccw.bmp");
	_image[7] = SDL_LoadBMP("./pics/q3cw.bmp");
	_image[8] = SDL_LoadBMP("./pics/empty.bmp");
	_image[9] = SDL_LoadBMP("./pics/whiteBall.bmp");
	_image[10] = SDL_LoadBMP("./pics/blackBall.bmp");
	_image[11] = SDL_LoadBMP("./pics/winner.bmp");
	_image[12] = SDL_LoadBMP("./pics/looser.bmp");
	_image[13] = SDL_LoadBMP("./pics/wait.bmp");

	int x, y, counter;

	counter = 0;
	x = 50;
	y = 50;

	for (unsigned int i = 0; i < 6; i += 1) {
		for (unsigned int j = 0; j < 6; j += 1) {
			_rectImage[counter].x = x;
			_rectImage[counter].y = y;
			counter++;
			x += 30;
		}
		y += 30;
		x = 50;
	}

	_rectBorder[0].x = 0;
	_rectBorder[0].y = 50;

	_rectBorder[1].x = 50;
	_rectBorder[1].y = 0;

	_rectBorder[2].x = 140;
	_rectBorder[2].y = 0;

	_rectBorder[3].x = 230;
	_rectBorder[3].y = 50;

	_rectBorder[4].x = 50;
	_rectBorder[4].y = 230;

	_rectBorder[5].x = 0;
	_rectBorder[5].y = 140;

	_rectBorder[6].x = 230;
	_rectBorder[6].y = 140;

	_rectBorder[7].x = 140;
	_rectBorder[7].y = 230;

	_rectWord.x = 50;
	_rectWord.y = 230;

}

void Engine::start() {
	_quit = false;
	_turn = true;
	_gameover = false;

	while(!_quit) {
		handle_input();
		do_render();
	}
}

void Engine::handle_input() {
	SDL_Event event;
	while(SDL_PollEvent(&event)) {
		switch(event.type) {
			case SDL_QUIT:
				_quit = true;
				break;
			case SDL_MOUSEBUTTONDOWN:
				if(!_gameover) mouse_button_down(event.button.button, event.motion.x, event.motion.y, event.motion.xrel, event.motion.yrel);
				break;
		}
	}
}

void Engine::do_render() {

	SDL_FillRect(_screen, NULL, _screen->format->Rmask | _screen->format->Gmask | _screen->format->Bmask );

	for (unsigned int i = 0; i < 36; i += 1) {
		if(_game->getPlayer(35 - i, true)) SDL_BlitSurface(_image[9],NULL,_screen,&_rectImage[i]);
		else if(_game->getPlayer(35 - i, false)) SDL_BlitSurface(_image[10],NULL,_screen,&_rectImage[i]);
		else SDL_BlitSurface(_image[8],NULL,_screen,&_rectImage[i]);
	}

	if(!_turn) {
		for (unsigned int i = 0; i < 8; i += 1)
			SDL_BlitSurface(_image[i],NULL,_screen,&_rectBorder[i]);		
	}

	if(_gameover) {
		if(_game->gameover(true)) {
			SDL_BlitSurface(_image[11],NULL,_screen,&_rectWord);		
		}
		else {
			SDL_BlitSurface(_image[12],NULL,_screen,&_rectWord);		
		}
	}

	SDL_Flip(_screen);
}

void Engine::mouse_button_down(const int& __button, const int& __x, const int& __y, const int& __relx, const int& __rely) {
	if(_turn) {

		if( (__x > 50) && (__y > 50) && (230 > __x) && (230 > __y)) {
			int x = (__x-50) / 30;
			int y = (__y-50) / 30;
			_game->setPlayer(35 - (y*6 + x), true);
			_turn = false;
		}
	}
	else {
			
		for (unsigned int i = 0; i < 8; i += 1) {
			int w = _image[i]->w;
			int h = _image[i]->h;
			if( (__x > _rectBorder[i].x) && (__y > _rectBorder[i].y) && ((_rectBorder[i].x + w) > __x) && ((_rectBorder[i].y + h) > __y)) {
				_game->turn(i/2, i%2);
				_turn = true;
			}			
		}

		if(_game->gameover()) {
			_gameover = true;
		}
		else {
			if(_turn) {
				do_render();
				SDL_BlitSurface(_image[13],NULL,_screen,&_rectWord);
				SDL_Flip(_screen);

				Search i;
				i.start(_game, 2, false);
			}
			if(_game->gameover()) {
				_gameover = true;
			}
		}

	}
}
